@@ -1,5 +1,3 @@
-context("Gather")
-
 test_that("gather all columns when ... is empty", {
   df <- data.frame(
     x = 1:5,
@@ -52,7 +50,10 @@ test_that("key preserves column ordering when factor_key = TRUE", {
 
 test_that("preserve class of input", {
   dat <- data.frame(x = 1:2)
-  dat %>% as_tibble() %>% gather() %>% expect_is("tbl_df")
+  dat %>%
+    as_tibble() %>%
+    gather() %>%
+    expect_s3_class("tbl_df")
 })
 
 test_that("additional inputs control which columns to gather", {
@@ -66,12 +67,16 @@ test_that("group_vars are kept where possible", {
   df <- tibble(x = 1, y = 1, z = 1)
 
   # Can't keep
-  out <- df %>% dplyr::group_by(x) %>% gather(key, val, x:z)
+  out <- df %>%
+    dplyr::group_by(x) %>%
+    gather(key, val, x:z)
   expect_equal(out, tibble(key = c("x", "y", "z"), val = 1))
 
   # Can keep
-  out <- df %>% dplyr::group_by(x) %>% gather(key, val, y:z)
-  expect_equal(dplyr::groups(out), list(quote(x)))
+  out <- df %>%
+    dplyr::group_by(x) %>%
+    gather(key, val, y:z)
+  expect_equal(dplyr::group_vars(out), "x")
 })
 
 test_that("overwrites existing vars", {
@@ -113,29 +118,36 @@ test_that("can gather all atomic vectors", {
   expect_equal(gather(df3, key, val, -y), gathered_key(1))
   expect_equal(gather(df4, key, val, -y), gathered_key("a"))
   expect_equal(gather(df5, key, val, -y), gathered_key(1 + 1i))
-
 })
 
 test_that("gather throws error for POSIXlt", {
   df <- data.frame(y = 1)
   df$x <- as.POSIXlt(Sys.time())
 
-  expect_error(gather(df, key, val, -x), "a POSIXlt")
-  expect_error(gather(df, key, val, -y), "a POSIXlt")
+  expect_snapshot({
+    (expect_error(gather(df, key, val, -x)))
+    (expect_error(gather(df, key, val, -y)))
+  })
 })
 
 test_that("gather throws error for weird objects", {
   df <- data.frame(y = 1)
   df$x <- expression(x)
-  expect_error(gather(df, key, val, -x), "atomic vector or list")
-  expect_error(gather(df, key, val, -y), "atomic vector or list")
+
+  expect_snapshot({
+    (expect_error(gather(df, key, val, -x)))
+    (expect_error(gather(df, key, val, -y)))
+  })
 
   e <- new.env(parent = emptyenv())
   e$x <- 1
   df <- data.frame(y = 1)
   df$x <- e
-  expect_error(gather(df, key, val, -x), "atomic vector or list")
-  expect_error(gather(df, key, val, -y), "atomic vector or list")
+
+  expect_snapshot({
+    (expect_error(gather(df, key, val, -x)))
+    (expect_error(gather(df, key, val, -y)))
+  })
 })
 
 
@@ -145,10 +157,7 @@ test_that("factors coerced to characters, not integers", {
     v2 = factor(letters[1:3])
   )
 
-  expect_warning(
-    out <- gather(df, k, v),
-    "attributes are not identical across measure variables"
-  )
+  expect_snapshot(out <- gather(df, k, v))
 
   expect_equal(out$v, c(1:3, letters[1:3]))
 })
@@ -165,18 +174,15 @@ test_that("common attributes are preserved", {
   df <- data.frame(date1 = Sys.Date(), date2 = Sys.Date() + 10)
   out <- gather(df, k, v)
 
-  expect_is(out$v, "Date")
+  expect_s3_class(out$v, "Date")
 })
 
 test_that("varying attributes are dropped with a warning", {
   df <- data.frame(
-    date1 = as.POSIXct(Sys.Date()),
-    date2 = Sys.Date() + 10
-  )
-  expect_warning(
-    gather(df, k, v),
-    "attributes are not identical across measure variables"
+    date1 = as.POSIXct("2019-01-01", tz = "UTC"),
+    date2 = as.Date("2019-01-01")
   )
+  expect_snapshot(gather(df, k, v))
 })
 
 test_that("gather preserves OBJECT bit on e.g. POSIXct", {
