@@ -47,7 +47,7 @@ test_that("`names_repair` happens after spec column reorganization (#1107)", {
     value = c(1, 2)
   )
 
-  out <- pivot_wider(df, names_repair = ~make.unique(.x))
+  out <- pivot_wider(df, names_repair = ~ make.unique(.x))
 
   expect_identical(out$test, c("a", "b"))
   expect_identical(out$test.1, c(1, NA))
@@ -205,6 +205,25 @@ test_that("expansion with `id_expand` and `names_expand` works with zero row dat
   expect_identical(res$b, c(NA_integer_, NA_integer_))
 })
 
+test_that("`build_wider_spec()` requires empty dots", {
+  df <- tibble(name = c("x", "y", "z"), value = 1:3)
+
+  expect_snapshot({
+    (expect_error(build_wider_spec(df, 1)))
+    (expect_error(build_wider_spec(df, name_prefix = "")))
+  })
+})
+
+test_that("`pivot_wider_spec()` requires empty dots", {
+  df <- tibble(name = c("x", "y", "z"), value = 1:3)
+  spec <- build_wider_spec(df)
+
+  expect_snapshot({
+    (expect_error(pivot_wider_spec(df, spec, 1)))
+    (expect_error(pivot_wider_spec(df, spec, name_repair = "check_unique")))
+  })
+})
+
 # column names -------------------------------------------------------------
 
 test_that("names_glue affects output names", {
@@ -215,7 +234,12 @@ test_that("names_glue affects output names", {
     b = 1:2
   )
 
-  spec <- build_wider_spec(df, x:y, a:b, names_glue = '{x}{y}_{.value}')
+  spec <- build_wider_spec(
+    df,
+    names_from = x:y,
+    values_from = a:b,
+    names_glue = "{x}{y}_{.value}"
+  )
   expect_equal(spec$.name, c("X1_a", "Y2_a", "X1_b", "Y2_b"))
 })
 
@@ -296,9 +320,9 @@ test_that("`names_expand` is validated", {
 test_that("can override default keys", {
   df <- tribble(
     ~row, ~name, ~var, ~value,
-    1,    "Sam", "age", 10,
-    2,    "Sam", "height", 1.5,
-    3,    "Bob", "age", 20,
+    1, "Sam", "age", 10,
+    2, "Sam", "height", 1.5,
+    3, "Bob", "age", 20,
   )
 
   pv <- df %>% pivot_wider(id_cols = name, names_from = var, values_from = value)
@@ -321,6 +345,30 @@ test_that("`id_cols = everything()` excludes `names_from` and `values_from`", {
   )
 })
 
+test_that("`id_cols` can't select columns from `names_from` or `values_from` (#1318)", {
+  df <- tibble(name = c("x", "y"), value = c(1, 2))
+
+  # And gives a nice error message!
+  expect_snapshot({
+    (expect_error(pivot_wider(df, id_cols = name, names_from = name, values_from = value)))
+    (expect_error(pivot_wider(df, id_cols = value, names_from = name, values_from = value)))
+  })
+})
+
+test_that("`id_cols` returns a tidyselect error if a column selection is OOB (#1318)", {
+  df <- tibble(name = c("x", "y"), value = c(1, 2))
+
+  expect_snapshot(
+    (expect_error(pivot_wider(df, id_cols = foo)))
+  )
+})
+
+test_that("named `id_cols` gives clear error (#1104)", {
+  df <- tibble(name = c("x", "y"), value = c(1, 2), x = 1, y = 2)
+
+  expect_snapshot(pivot_wider(df, id_cols = c(z = x)), error = TRUE)
+})
+
 test_that("pivoting a zero row data frame drops `names_from` and `values_from` (#1249)", {
   df <- tibble(key = character(), name = character(), value = integer())
 
@@ -489,7 +537,7 @@ test_that("values_fn can be a single function", {
 
 test_that("values_fn can be an anonymous function (#1114)", {
   df <- tibble(a = c(1, 1, 2), key = c("x", "x", "x"), val = c(1, 10, 100))
-  pv <- pivot_wider(df, names_from = key, values_from = val, values_fn = ~sum(.x))
+  pv <- pivot_wider(df, names_from = key, values_from = val, values_fn = ~ sum(.x))
   expect_equal(pv$x, c(11, 100))
 })
 
@@ -528,7 +576,7 @@ test_that("can fill in missing cells", {
 
 test_that("values_fill only affects missing cells", {
   df <- tibble(g = c(1, 2), names = c("x", "y"), value = c(1, NA))
-  out <- pivot_wider(df, names_from = names, values_from = value, values_fill = 0 )
+  out <- pivot_wider(df, names_from = names, values_from = value, values_fill = 0)
   expect_equal(out$y, c(0, NA))
 })
 
@@ -576,10 +624,10 @@ test_that("column order in output matches spec", {
 
   # deliberately create weird order
   sp <- tribble(
-    ~hw, ~.value,  ~.name,
+    ~hw, ~.value, ~.name,
     "hw1", "mark", "hw1_mark",
-    "hw1", "pr",   "hw1_pr",
-    "hw2", "pr",   "hw2_pr",
+    "hw1", "pr", "hw1_pr",
+    "hw2", "pr", "hw2_pr",
     "hw2", "mark", "hw2_mark",
   )
 
@@ -618,7 +666,7 @@ test_that("`unused_fn` works with anonymous functions", {
     value = c(1, 2, 3, 4)
   )
 
-  res <- pivot_wider(df, id_cols = id, unused_fn = ~mean(.x, na.rm = TRUE))
+  res <- pivot_wider(df, id_cols = id, unused_fn = ~ mean(.x, na.rm = TRUE))
   expect_identical(res$unused, c(1, 3.5))
 })
 
@@ -647,7 +695,7 @@ test_that("`unused_fn` works with expanded key from `id_expand`", {
   expect_identical(res$id, factor(1:3))
   expect_identical(res$unused, c(2, 4, NA))
 
-  res <- pivot_wider(df, id_cols = id, id_expand = TRUE, unused_fn = ~sum(is.na(.x)))
+  res <- pivot_wider(df, id_cols = id, id_expand = TRUE, unused_fn = ~ sum(is.na(.x)))
   expect_identical(res$unused, c(0L, 0L, 1L))
 })
 
@@ -685,6 +733,14 @@ test_that("can't fill implicit missings in unused column with `values_fill`", {
   expect_identical(res$unused, list(c(1, 2), c(4, 3), NA_real_))
 })
 
+test_that("`values_fill` is validated", {
+  df <- tibble(name = "a", value = 1)
+
+  expect_snapshot(
+    (expect_error(pivot_wider(df, values_fill = 1:2)))
+  )
+})
+
 test_that("`unused_fn` is validated", {
   df <- tibble(id = 1, unused = 1, name = "a", value = 1)
 
@@ -692,3 +748,66 @@ test_that("`unused_fn` is validated", {
     (expect_error(pivot_wider(df, id_cols = id, unused_fn = 1)))
   )
 })
+
+# deprecated ---------------------------------------------------------------
+
+test_that("`id_cols` has noisy compat behavior (#1353)", {
+  df <- tibble(
+    id = c(1, 2),
+    id2 = c(3, 4),
+    name = c("a", "b"),
+    value = c(5, 6)
+  )
+
+  # Noisy
+  expect_snapshot({
+    out <- pivot_wider(df, id)
+  })
+
+  # Silent
+  expect_snapshot({
+    expect <- pivot_wider(df, id_cols = id)
+  })
+
+  expect_identical(out, expect)
+})
+
+test_that("`id_cols` compat behavior doesn't trigger if `id_cols` is specified too", {
+  df <- tibble(
+    id = c(1, 2),
+    id2 = c(3, 4),
+    name = c("a", "b"),
+    value = c(5, 6)
+  )
+
+  expect_snapshot(error = TRUE, {
+    pivot_wider(df, id, id_cols = id2)
+  })
+})
+
+test_that("`id_cols` compat behavior doesn't trigger if multiple `...` are supplied", {
+  df <- tibble(
+    id = c(1, 2),
+    id2 = c(3, 4),
+    name = c("a", "b"),
+    value = c(5, 6)
+  )
+
+  expect_snapshot(error = TRUE, {
+    pivot_wider(df, id, id2)
+  })
+})
+
+test_that("`id_cols` compat behavior doesn't trigger if named `...` are supplied", {
+  df <- tibble(
+    id = c(1, 2),
+    id2 = c(3, 4),
+    name = c("a", "b"),
+    value = c(5, 6)
+  )
+
+  expect_snapshot(error = TRUE, {
+    pivot_wider(df, ids = id)
+  })
+})
+
